@@ -9,8 +9,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkPort.h,v 1.3.2.1 2004/01/01 00:34:56 das Exp $
  */
 
 #ifndef _TKPORT
@@ -26,9 +24,7 @@
 #if defined(__WIN32__) || defined(_WIN32)
 #   include "tkWinPort.h"
 #else
-#   if defined(MAC_TCL)
-#	include "tkMacPort.h"
-#   elif defined(MAC_OSX_TK)
+#   if defined(MAC_OSX_TK)
 #	include "tkMacOSXPort.h"
 #   else
 #	include "tkUnixPort.h"
