@@ -129,6 +129,7 @@
 #endif
 
 #ifdef __CYGWIN__
+#   include "tkIntXlibDecls.h"
 #   define UINT unsigned int
 #   define HWND void *
 #   define HDC void *
@@ -142,9 +143,6 @@
 #   define LPARAM void *
 #   define LRESULT void *
 
-    extern int TkPutImage(unsigned long *, int, Display *, Drawable,
-	    GC, XImage *, int, int, int, int, unsigned int, unsigned int);
-
 #else /* !__CYGWIN__ */
     /*
      * The TkPutImage macro strips off the color table information, which isn't
