@@ -33,7 +33,7 @@
 
 bind Listbox <1> {
     if {[winfo exists %W]} {
-	tk::ListboxBeginSelect %W [%W index @%x,%y]
+	tk::ListboxBeginSelect %W [%W index @%x,%y] 1
     }
 }
 
@@ -118,7 +118,7 @@ bind Listbox <Control-Home> {
     %W see 0
     %W selection clear 0 end
     %W selection set 0
-    event generate %W <<ListboxSelect>>
+    tk::FireListboxSelectEvent %W
 }
 bind Listbox <Shift-Control-Home> {
     tk::ListboxDataExtend %W 0
@@ -128,7 +128,7 @@ bind Listbox <Control-End> {
     %W see end
     %W selection clear 0 end
     %W selection set end
-    event generate %W <<ListboxSelect>>
+    tk::FireListboxSelectEvent %W
 }
 bind Listbox <Shift-Control-End> {
     tk::ListboxDataExtend %W [%W index end]
@@ -142,6 +142,9 @@ bind Listbox <<Copy>> {
 bind Listbox <space> {
     tk::ListboxBeginSelect %W [%W index active]
 }
+bind Listbox <<Invoke>> {
+    tk::ListboxBeginSelect %W [%W index active]
+}
 bind Listbox <Select> {
     tk::ListboxBeginSelect %W [%W index active]
 }
@@ -160,7 +163,7 @@ bind Listbox <Control-slash> {
 bind Listbox <Control-backslash> {
     if {[%W cget -selectmode] ne "browse"} {
 	%W selection clear 0 end
-	event generate %W <<ListboxSelect>>
+        tk::FireListboxSelectEvent %W
     }
 }
 
@@ -176,6 +179,7 @@ bind Listbox <B2-Motion> {
 # The MouseWheel will typically only fire on Windows and Mac OS X.
 # However, someone could use the "event generate" command to produce
 # one on other platforms.
+
 if {[tk windowingsystem] eq "aqua"} {
     bind Listbox <MouseWheel> {
         %W yview scroll [expr {- (%D)}] units
@@ -224,7 +228,7 @@ if {"x11" eq [tk windowingsystem]} {
 # el -		The element for the selection operation (typically the
 #		one under the pointer).  Must be in numerical form.
 
-proc ::tk::ListboxBeginSelect {w el} {
+proc ::tk::ListboxBeginSelect {w el {focus 1}} {
     variable ::tk::Priv
     if {[$w cget -selectmode] eq "multiple"} {
 	if {[$w selection includes $el]} {
@@ -239,7 +243,11 @@ proc ::tk::ListboxBeginSelect {w el} {
 	set Priv(listboxSelection) {}
 	set Priv(listboxPrev) $el
     }
-    event generate $w <<ListboxSelect>>
+    tk::FireListboxSelectEvent $w
+    # check existence as ListboxSelect may destroy us
+    if {$focus && [winfo exists $w] && [$w cget -state] eq "normal"} {
+	focus $w
+    }
 }
 
 # ::tk::ListboxMotion --
@@ -263,7 +271,7 @@ proc ::tk::ListboxMotion {w el} {
 	    $w selection clear 0 end
 	    $w selection set $el
 	    set Priv(listboxPrev) $el
-	    event generate $w <<ListboxSelect>>
+	    tk::FireListboxSelectEvent $w
 	}
 	extended {
 	    set i $Priv(listboxPrev)
@@ -294,7 +302,7 @@ proc ::tk::ListboxMotion {w el} {
 		incr i -1
 	    }
 	    set Priv(listboxPrev) $el
-	    event generate $w <<ListboxSelect>>
+	    tk::FireListboxSelectEvent $w
 	}
     }
 }
@@ -345,7 +353,7 @@ proc ::tk::ListboxBeginToggle {w el} {
 	} else {
 	    $w selection set $el
 	}
-	event generate $w <<ListboxSelect>>
+	tk::FireListboxSelectEvent $w
     }
 }
 
@@ -397,7 +405,7 @@ proc ::tk::ListboxUpDown {w amount} {
 	browse {
 	    $w selection clear 0 end
 	    $w selection set active
-	    event generate $w <<ListboxSelect>>
+	    tk::FireListboxSelectEvent $w
 	}
 	extended {
 	    $w selection clear 0 end
@@ -405,7 +413,7 @@ proc ::tk::ListboxUpDown {w amount} {
 	    $w selection anchor active
 	    set Priv(listboxPrev) [$w index active]
 	    set Priv(listboxSelection) {}
-	    event generate $w <<ListboxSelect>>
+	    tk::FireListboxSelectEvent $w
 	}
     }
 }
@@ -477,7 +485,7 @@ proc ::tk::ListboxCancel w {
     }
     set first [$w index anchor]
     set last $Priv(listboxPrev)
-    if { $last eq "" } {
+    if {$last eq ""} {
 	# Not actually doing any selection right now
 	return
     }
@@ -493,7 +501,7 @@ proc ::tk::ListboxCancel w {
 	}
 	incr first
     }
-    event generate $w <<ListboxSelect>>
+    tk::FireListboxSelectEvent $w
 }
 
 # ::tk::ListboxSelectAll
@@ -513,5 +521,19 @@ proc ::tk::ListboxSelectAll w {
     } else {
 	$w selection set 0 end
     }
-    event generate $w <<ListboxSelect>>
+    tk::FireListboxSelectEvent $w
+}
+
+# ::tk::FireListboxSelectEvent
+#
+# Fire the <<ListboxSelect>> event if the listbox is not in disabled
+# state.
+#
+# Arguments:
+# w -		The listbox widget.
+
+proc ::tk::FireListboxSelectEvent w {
+    if {[$w cget -state] eq "normal"} {
+        event generate $w <<ListboxSelect>>
+    }
 }
